include "GenericIndustry.gs"

//
//
//
class CoalMine isclass GenericIndustry
{
  ProductQueue coalOutQueue, dieselInQueue;
  bool scriptletEnabled = true;
  //bool nodiesel = false;
  bool animating = false;
  bool processing = false;

  // Track if they only supply some of the logs that were requested in the waybill.
  int dieselWBRemain = 0;

  //
  // Displays statistics on current levels of various queus in pop-up browser window
  //
  thread void ViewDetails(void)
  {
    if (!info)
    {
      info = Constructors.NewBrowser();
      
      info.LoadHTMLString(HTMLWindow.GetCompleteIndustryViewDetailsHTMLCode(me, scriptletEnabled));
	  info.SetWindowRect(100, 80, 500, 545);
    }
  }
  
  //
  //
  //

  bool TriggerSupportsMovingLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.LOAD_COMMAND))
      if (triggerName == "out_load0" or triggerName == "out_load1" or triggerName == "out_load2" or triggerName == "out_load3")
        return true;
    
    return false;
  }


  void PerformMovingLoad(Vehicle vehicle, string triggerName)
  {
    //
    // OUTPUT trigger - load 
    //
    float speed = vehicle.GetVelocity();

    if (speed > -5.0f and speed < 5.0f)
    {
      int coalAvailable = coalOutQueue.GetQueueCount();
      if (triggerName == "out_load0")
      {
        SendMessage(me, "pfx", "+2");
        //World.PlaySound("coal_load.wav");
        LoadingReport report = CreateLoadingReport(coalOutQueue, coalAvailable);
        vehicle.LoadProduct(report);
        SendMessage(me, "pfx", "-2");
      }
      if (triggerName == "out_load1")
      {
        SendMessage(me, "pfx", "+3");
        //World.PlaySound("coal_load.wav");
        LoadingReport report = CreateLoadingReport(coalOutQueue, coalAvailable);
        vehicle.LoadProduct(report);
        SendMessage(me, "pfx", "-3");
      }
      if (triggerName == "out_load2")
      {
        SendMessage(me, "pfx", "+4");
        //World.PlaySound("coal_load.wav");
        LoadingReport report = CreateLoadingReport(coalOutQueue, coalAvailable);
        vehicle.LoadProduct(report);
        SendMessage(me, "pfx", "-4");
      }
      if (triggerName == "out_load3")
      {
        SendMessage(me, "pfx", "+5");
        //World.PlaySound("coal_load.wav");
        LoadingReport report = CreateLoadingReport(coalOutQueue, coalAvailable);
        vehicle.LoadProduct(report);
        SendMessage(me, "pfx", "-5");
      }
    }
  }


  //
  //
  //

  bool TriggerSupportsStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.UNLOAD_COMMAND))
    {
      bool vehicleToTrain = vehicle.GetFacingRelativeToTrain();
      int direction = vehicle.GetRelationToTrack(me, "in_track0");
      if (!vehicleToTrain)
	      direction = -direction;

      // Are we up to the furthest trigger away from the side we entered for diesel?
      if (direction == Vehicle.DIRECTION_BACKWARD and triggerName == "in_load0")
        return true;
      if (direction == Vehicle.DIRECTION_FORWARD and triggerName == "in_load3")
        return true;

      // If the train has already stopped, then fall thru and allow this load as well
      if (triggerName == "in_load0" or triggerName == "in_load1" or triggerName == "in_load2" or triggerName == "in_load3")
      {
        if (vehicle.GetMyTrain().IsStopped())
          return true;
      }
    }
    
    return false;
  }


  void PerformStoppedLoad(Vehicle vehicle, string triggerName)
  {
    int spaceAvailable = dieselInQueue.GetQueueSpace();
    LoadingReport report = CreateUnloadingReport(dieselInQueue, spaceAvailable);

    int direction = vehicle.GetRelationToTrack(me, "in_track0");
    if (direction == Vehicle.DIRECTION_FORWARD)
      report.sideFlags = LoadingReport.LEFT_SIDE;
    else if (direction == Vehicle.DIRECTION_BACKWARD)
      report.sideFlags = LoadingReport.RIGHT_SIDE;

    vehicle.UnloadProduct(report);

    // Ensure we are tracking this if we are waiting for a way bill to complete in its entirety
    if (dieselWBRemain > 0)
      dieselWBRemain = dieselWBRemain - report.amount;

    /*if (report.amount > 0)
    {
      nodiesel = false;
      SetMeshAnimationFrame("default", 2);
      StartMeshAnimationLoop("default");
    }*/
  }


  //
  // ?
  //
  thread void CoalMain(void)
  {
    Message msg;

    wait()
    {
      on "Scriptlet-Enabled", "1":
      {
        if (!scriptletEnabled)
        {
          scriptletEnabled = true;
          SetProcessEnabled("coal_consumer", true);
        }
        continue;
      }

      // ? Power station is providing electricity, if not already running. start the 
      on "Scriptlet-Enabled", "0":
      {
        if (scriptletEnabled)
        {
          scriptletEnabled = false;
          SetProcessEnabled("coal_consumer", false);
        }
        continue;
      }

      // logs_consumer process in lumber mill has started, so active smoke stack particles and
      // start the animation (forklift & conveyor)
      on "Process-Start", "coal_consumer":
        //Interface.Print("CoalMine.LumberMain(): Process-Start:coal_consumer message received, starting default animation amd smoke stack particles");

        SendMessage(me, "pfx", "+0+1");

        if (animating) // only start animating if animation isn't already running
          StartMeshAnimationLoop("default");

        continue;

      // logs_consumer process has stopped, disable particle effect and stop animation
      on "Process-Stop", "coal_consumer":
        //Interface.Print("CoalMine.LumberMain(): Process-Stop:coal_consumer message received, stopping default animation and smoke stack particles");

        SendMessage(me, "pfx", "-0-1");
        StopMeshAnimation("default");
        continue;


      // ?
      on "Animation-Event", "animstop":
      {
        //if (!(GetProcessStarted("logs_consumer") and GetProcessEnabled("logs_consumer")))
        //Interface.Print("Animation-Event: animstop message received, stopping default animation!");
        
        StopMeshAnimation("default");

        // make sure we didn't overshoot the end of the animation
        SetMeshAnimationFrame("default", 2);

        
        // it's actually possible to receive the Animation-Event message multiple times, even 
        // though we tell the game engine to stop the animation. this can occur if the frame-rate
        // is low and the animation repeats multiple times inside one frame. it's important that we
        // check for this situation, otherwise a script exception will result.
        if (animating)
        {
          animating = false;

          if (!processing)
          {
            // we are making the assumption that there is only one process, "coal_consumer"
            // if there were more, we would have to be careful to finish the correct process here
            PerformProcessOutput("coal_consumer");
            PerformProcessFinished("coal_consumer");
          }
        }

        continue;
      }
    
    }
  }


  //
  // Called by the game once when a process is ready to start (see Industry.gs)
  //
  void NotifyProcessStarted(string processName)
  {
    //Interface.Print("CoalMine.NotifyProcessFinished: Process Started");

    if (scriptletEnabled) // only when power is running
    {

      if (PerformProcessInput(processName))
      {
        //Interface.Print("NotifyProcessStarted: Starting the default animation!");

        // we are making the assumption that there is only one process, "logs_consumer"
        SetMeshAnimationFrame("default", 2);
        StartMeshAnimationLoop("default");

        animating = true;
        processing = true;

        PerformProcessStarted(processName);
      }
      else
        PerformProcessCancelled(processName);

    }

  }

  
  // called by the game once when a process is ready to stop (see Industry.gs)
  void NotifyProcessFinished(string processName)
  {
    //Interface.Print("CoalMine.NotifyProcessFinished: Process Finished");

    if (scriptletEnabled)
    {
      processing = false;

      if (!animating)
      {
        PerformProcessOutput(processName);
        PerformProcessFinished(processName);
      }
    }

    //if (GetProcessInput(processName, dieselInQueue, GetAsset().FindAsset("diesel")) <= 0)
    //{
      //nodiesel = true;
      //StopMeshAnimation("default");
      //SetMeshAnimationFrame("default", 1098);
    //}
  }



  //
  //
  //
  public void Init(void)
  {
    inherited();

    usePipeAnimation = true;

    coalOutQueue = GetQueue("coal_out");
    dieselInQueue = GetQueue("diesel_in");

    AddAssetToIndustryProductInfo("diesel", "diesel_in", "coal_consumer", true);
    AddAssetToIndustryProductInfo("coal", "coal_out", "coal_consumer", false);

    // Enabled or disabled on startup? (Depends on if we have fuel! :D)
    /*if (dieselInQueue.GetQueueCount() > 0)
    {
      nodiesel = false;
      SetMeshAnimationFrame("default", 2);
      StartMeshAnimationLoop("default");
    }
    else
    {
      nodiesel = true;
      StopMeshAnimation("default");
      SetMeshAnimationFrame("default", 1098);
    }*/
    
    CoalMain();
  }


  public Requirement[] GetRequirements(void)
  {
    Requirement[] ret = new Requirement[0];

    if (dieselInQueue.GetQueueCount() < 77600 or dieselWBRemain > 0) // Approx 25% of full
    {
      ResourceRequirement req = new ResourceRequirement();

      req.resource = dieselInQueue.GetProductFilter().GetProducts()[0];
      // This is how many we have asked for. Wait till it is fullfilled, 
      // if we are not already waiting for a waybill to be completed.
      req.amount = 232800;        // 6 tank cars
      if (dieselInQueue.GetQueueCount() < 77600 and dieselWBRemain == 0)
        dieselWBRemain = 232800;
      req.dst = me;
      req.dstQueue = dieselInQueue;

      ret[ret.size()] = req;
    }

    return ret;
  }

  public void AppendDriverDestinations(string[] destNames, string[] destTracks)
  {
    destNames[destNames.size()] = "Coal load bay #1";
    destTracks[destTracks.size()] = "out_track0";
    
    destNames[destNames.size()] = "Coal load bay #2";
    destTracks[destTracks.size()] = "out_track1";
    
    destNames[destNames.size()] = "Coal load bay #3";
    destTracks[destTracks.size()] = "out_track2";
    
    destNames[destNames.size()] = "Coal load bay #4";
    destTracks[destTracks.size()] = "out_track3";
    
    destNames[destNames.size()] = "Diesel unload bay";
    destTracks[destTracks.size()] = "in_track0";
  }
};