//
// forestry.gs
//
//  Copyright (C) 2003 Auran Developments Pty Ltd
//  All Rights Reserved.
//

include "GenericIndustry.gs"


//
// Refinery industry
//
class forestry isclass GenericIndustry
{
  ProductQueue logsOutQueue;
  bool animating = false;
  bool processing = false;
  bool scriptletEnabled = true;
  bool waitingForOutput = false;

  
  //
  //
  //
  thread void ViewDetails(void)
  {
    if (!info)
    {
      info = Constructors.NewBrowser();
      
      info.LoadHTMLString(HTMLWindow.GetCompleteIndustryViewDetailsHTMLCode(me, scriptletEnabled));
	  info.SetWindowRect(100, 80, 500, 545);
    }
  }



  bool TriggerSupportsStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.LOAD_COMMAND))
	    if (triggerName == "out_load0")
		    return true;
	  
	  return false;
  }

  void PerformStoppedLoad(Vehicle vehicle, string triggerName)
  {
    int logsAvailable = logsOutQueue.GetQueueCount();
    LoadingReport report = CreateLoadingReport(logsOutQueue, logsAvailable);
    vehicle.LoadProduct(report);
  }

      
  bool HandleTrainUnloadCommand(Train train)
  {
	  return false;
  }

  
  //
  //
  //
  thread void TreeLoppingMain(void)
  {
    wait()
    {
      // logs_consumer process in lumber mill has started, so active smoke stack particles and
      // start the animation (forklift & conveyor)
      on "Process-Start", "tree_lopping":
      {
        processing = true;

        if (!animating) // only start animating if animation isn't already running
          StartMeshAnimationLoop("default");

        animating = true;

        continue;
      }

      on "Animation-Event", "animfin":
      {
        // it's actually possible to receive the Animation-Event message multiple times, even 
        // though we tell the game engine to stop the animation. this can occur if the frame-rate
        // is low and the animation repeats multiple times inside one frame. it's important that we
        // check for this situation, otherwise a script exception will result.
        if (waitingForOutput)
        {
          PerformProcessFinished("tree_lopping");
          waitingForOutput = false;
          
					//autoLoader.ReSendStoppedMessages(me);
					InnerReEnter(-1);

        }
        // If we are no longer processing, then stop the animation
        StopMeshAnimation("default");
        animating = false;

        continue;
      }

      on "Animation-Event", "longdrop":
      {
        // it's actually possible to receive the Animation-Event message multiple times, even 
        // though we tell the game engine to stop the animation. this can occur if the frame-rate
        // is low and the animation repeats multiple times inside one frame. it's important that we
        // check for this situation, otherwise a script exception will result.
        // we are making the assumption that there is only one process, "tree_lopping"
        // if there were more, we would have to be careful to finish the correct process here
        if (waitingForOutput)
          PerformProcessOutput("tree_lopping");

        continue;
      }

      // ?
      on "Scriptlet-Enabled", "1":
      {
        if (!scriptletEnabled)
        {
          scriptletEnabled = true;
          SetProcessEnabled("tree_lopping", true);
          StartMeshAnimationLoop("default");
        }
        continue;
      }

      // ?
      on "Scriptlet-Enabled", "0":
      {
        if (scriptletEnabled)
        {
          scriptletEnabled = false;
          processing = false;
          animating = false;
          StopMeshAnimation("default");
          SetProcessEnabled("tree_lopping", false);
        }
        continue;
      }
    }
  }
  
  
  //
  // Called by the game once when a process is ready to start (see Industry.gs)
  //
  void NotifyProcessStarted(string processName)
  {
    //Interface.Print("NotifyProcessStarted called in lumber mill!");

    if (scriptletEnabled) // only when power is running
    {

      if (PerformProcessInput(processName))
      {
        //Interface.Print("NotifyProcessStarted: Starting the default animation!");

        // we are making the assumption that there is only one process, "logs_consumer"
        SetMeshAnimationFrame("default", 0);
        StartMeshAnimationLoop("default");

        animating = true;
        processing = true;
        waitingForOutput = true;

        PerformProcessStarted(processName);
      }
      else
      {
        PerformProcessCancelled(processName);
        processing = false;
      }

    }

  }

  
  // called by the game once when a process is ready to stop (see Industry.gs)
  void NotifyProcessFinished(string processName)
  {
    waitingForOutput = true;
    //processing = false;

/*    if (!animating)
    {
      PerformProcessOutput(processName);
      PerformProcessFinished(processName);
    }*/
  }


  //
  //
  //
  public void Init(void)
  {
		inherited();

    // initialize queues
    logsOutQueue = GetQueue("logs_out");

    AddAssetToIndustryProductInfo("logs", "logs_out", "tree_lopping", false);

    SetProcessEnabled("tree_lopping", true);
    StartMeshAnimationLoop("default");
    animating = true;
    processing = true;


    TreeLoppingMain();
  }

  public void AppendDriverDestinations(string[] destNames, string[] destTracks)
  {
    destNames[destNames.size()] = "Log pickup";
    destTracks[destTracks.size()] = "out_track0";
    
  }


};