include "GenericIndustry.gs"

//
// ?
//
class lumber_mill isclass GenericIndustry
{
  ProductQueue logsInQueue, chipsOutQueue, lumberOutQueue;
  bool animating = false;         // is the forklift/conveyor belt animation currently running?
  bool processing = false;        // is the log consumer process currently running?
  bool scriptletEnabled = true;  // state of power supply (must be true for this mill to run)

  // Track if they only supply some of the logs that were requested in the waybill.
  int logWBRemain = 0;


  //
  // Displays statistics on current levels of various queus in pop-up browser window
  //
  thread void ViewDetails(void)
  {
    if (!info)
    {
      info = Constructors.NewBrowser();
      
      info.LoadHTMLString(HTMLWindow.GetCompleteIndustryViewDetailsHTMLCode(me, scriptletEnabled));
	  info.SetWindowRect(100, 80, 500, 545);
    }
  }



  bool TriggerSupportsMovingLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.LOAD_COMMAND))
      if (triggerName == "woodchips_t")
        return true;
    
    return false;
  }

  void PerformMovingLoad(Vehicle vehicle, string triggerName)
  {
    if (scriptletEnabled)
    {
      float speed = vehicle.GetVelocity();

      if (speed > -5.0f and speed < 5.0f)
      {
        int available = chipsOutQueue.GetQueueCount();

        SendMessage(me, "pfx", "+1+2");
        LoadingReport report = CreateLoadingReport(chipsOutQueue, available);
        vehicle.LoadProduct(report);

        SendMessage(me, "pfx", "-1-2");
      }
    }
  }


  bool TriggerSupportsStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.LOAD_COMMAND))
      if (triggerName == "lumber_t")
        return true;
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.UNLOAD_COMMAND))
      if (triggerName == "logs_t")
        return true;

    return false;
  }

  void PerformStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (triggerName == "lumber_t")
    {
      int available = lumberOutQueue.GetQueueCount();
      LoadingReport report = CreateLoadingReport(lumberOutQueue, available);
      vehicle.LoadProduct(report);
    }

    else if (triggerName == "logs_t")
    {
      int spaceAvailable = logsInQueue.GetQueueSpace();
      LoadingReport report = CreateUnloadingReport(logsInQueue, spaceAvailable);
      vehicle.UnloadProduct(report);

      // Ensure we are tracking this if we are waiting for a way bill to complete in its entirety
      if (logWBRemain > 0)
        logWBRemain = logWBRemain - report.amount;
    }
  }



  //
  // Thread that runs this lumber mill scriplet.
  //
  thread void LumberMain(void)
  {
    Message msg;
    Vehicle vehicle;
    string triggerName;

    //
    wait()
    {

      // logs_consumer process in lumber mill has started, so active smoke stack particles and
      // start the animation (forklift & conveyor)
      on "Process-Start", "logs_consumer":
        //Interface.Print("LumberMain(): Process-Start:logs_consumer message received, starting default animation amd smoke stack particles");

        SendMessage(me, "pfx", "+0");

        StartMeshAnimationLoop("default");

        continue;

      // logs_consumer process has stopped, disable particle effect and stop animation
      on "Process-Stop", "logs_consumer":
        //Interface.Print("LumberMain(): Process-Stop message received, stopping default animation and smoke stack particles");

        SendMessage(me, "pfx", "-0");
        StopMeshAnimation("default");
        continue;


      // ?
      on "Animation-Event", "animstop":
    {
        //if (!(GetProcessStarted("logs_consumer") and GetProcessEnabled("logs_consumer")))
        //Interface.Print("Animation-Event,animstop message received, stopping default animation!");
        //Interface.Print("Animation-Event,animstop message received, stopping default animation!");
        
        StopMeshAnimation("default");

        // make sure we didn't overshoot the end of the animation
        SetMeshAnimationFrame("default", 999.5);

        
        // it's actually possible to receive the Animation-Event message multiple times, even 
        // though we tell the game engine to stop the animation. this can occur if the frame-rate
        // is low and the animation repeats multiple times inside one frame. it's important that we
        // check for this situation, otherwise a script exception will result.
        if (animating)
        {
          animating = false;

          if (!processing)
          {
            // we are making the assumption that there is only one process, "logs_consumer"
            // if there were more, we would have to be careful to finish the correct process here
            PerformProcessOutput("logs_consumer");
            PerformProcessFinished("logs_consumer");
            
            //autoLoader.ReSendStoppedMessages(me);
            InnerReEnter(-1);
          }
        }

        continue;
    }

      // Messages with a major of "Scriptlet-Enabled" are sent by the PowerStationRule scriplet
      // when the power station's coal consumption process has started (minor = "1") or stopped
      // (minor = "0").

      // Power station is providing electricity, if not already running. start the 
      on "Scriptlet-Enabled", "1":
      {
        //Interface.Print("Enable script ");
        if (!scriptletEnabled)
        {
          scriptletEnabled = true;
          SetProcessEnabled("logs_consumer", true);
        }
        continue;
      }

      // ? Power station is providing electricity, if not already running. start the 
      on "Scriptlet-Enabled", "0":
      {
        //Interface.Print("Disable script ");
        if (scriptletEnabled)
        {
          scriptletEnabled = false;
          SetProcessEnabled("logs_consumer", false);
        }
        continue;
      }
    }
  }


  //
  // Called by the game once when a process is ready to start (see Industry.gs)
  //
  void NotifyProcessStarted(string processName)
  {
    //Interface.Print("NotifyProcessStarted called in lumber mill!");

    if (scriptletEnabled) // only when power is running
    {

      if (PerformProcessInput(processName))
      {
        //Interface.Print("NotifyProcessStarted: Starting the default animation!");

        // we are making the assumption that there is only one process, "logs_consumer"
        SetMeshAnimationFrame("default", 0);
        StartMeshAnimationLoop("default");

        animating = true;
        processing = true;

        PerformProcessStarted(processName);
      }
      else
        PerformProcessCancelled(processName);

    }

  }

  
  // called by the game once when a process is ready to stop (see Industry.gs)
  void NotifyProcessFinished(string processName)
  {

    processing = false;

    if (!animating)
    {
      PerformProcessOutput(processName);
      PerformProcessFinished(processName);
    }
  }


  //
  //
  //
  public void Init(void)
  {
    inherited();

    logsInQueue = GetQueue("logs_q");
    chipsOutQueue = GetQueue("woodchips_q");
    lumberOutQueue = GetQueue("lumber_q");

    AddAssetToIndustryProductInfo("logs", "logs_q", "logs_consumer", true);
    AddAssetToIndustryProductInfo("woodchips", "woodchips_q", "logs_consumer", false);
    AddAssetToIndustryProductInfo("lumber", "lumber_q", "logs_consumer", false);

    // 
    StopMeshAnimation("default");
    SetMeshAnimationFrame("default", 999);
    

    // Start main thread to process messages and run the lumber mill.
    LumberMain();
  }


  public Requirement[] GetRequirements(void)
  {
    Requirement[] ret = new Requirement[0];

    if (logsInQueue.GetQueueCount() < 10 or logWBRemain > 0)
    {
      ResourceRequirement req = new ResourceRequirement();

      req.resource = logsInQueue.GetProductFilter().GetProducts()[0];
      
      // This is how many we have asked for. Wait till it is fullfilled, 
      // if we are not already waiting for a waybill to be completed.
      req.amount = 25;
      if (logsInQueue.GetQueueCount() < 10 and logWBRemain == 0)
        logWBRemain = 25;

      req.dst = me;
      req.dstQueue = logsInQueue;

      ret[ret.size()] = req;
    }

    return ret;
  }

  public void AppendDriverDestinations(string[] destNames, string[] destTracks)
  {
    destNames[destNames.size()] = "Log Drop off";
    destTracks[destTracks.size()] = "Log_Depot";
    
    destNames[destNames.size()] = "Woodchips pickup";
    destTracks[destTracks.size()] = "Woodchips_Depot";

    destNames[destNames.size()] = "Lumber pickup";
    destTracks[destTracks.size()] = "Lumber_Depot";
  
  }

};




