include "GenericIndustry.gs"


//
// Refinery industry
//
class Refinery isclass GenericIndustry
{
  ProductQueue crudeOilInQueue, dieselOutQueue, avgasOutQueue, petrolOutQueue;

  bool scriptletEnabled = true;

  // Track if they only supply some of the logs that were requested in the waybill.
  int oilWBRemain = 0;
  
  //
  //
  //
  thread void ViewDetails(void)
  {
    if (!info)
    {
      info = Constructors.NewBrowser();

      info.LoadHTMLString(HTMLWindow.GetCompleteIndustryViewDetailsHTMLCode(me, scriptletEnabled));
	  info.SetWindowRect(100, 80, 500, 545);
    }

  }

  //
  //
  //

  bool TriggerSupportsStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.LOAD_COMMAND))
    {
      bool vehicleToTrain = vehicle.GetFacingRelativeToTrain();
      int direction = vehicle.GetRelationToTrack(me, "outrack");
      if (!vehicleToTrain)
	      direction = -direction;

      if (direction == Vehicle.DIRECTION_BACKWARD and triggerName == "fuel_loading3")
        return true;
      if (direction == Vehicle.DIRECTION_FORWARD and triggerName == "fuel_loading0")
        return true;

      direction = vehicle.GetRelationToTrack(me, "intrack");
      if (!vehicleToTrain)
	      direction = -direction;

      if (triggerName == "fuel_loading0" or triggerName == "fuel_loading1" or triggerName == "fuel_loading2" or triggerName == "fuel_loading3")
      {
        if (vehicle.GetMyTrain().IsStopped())
          return true;
      }    
    }

    if (itc.IsTrainCommand(vehicle.GetMyTrain(), Industry.UNLOAD_COMMAND))
    {
      bool vehicleToTrain = vehicle.GetFacingRelativeToTrain();
      int direction = vehicle.GetRelationToTrack(me, "intrack");
      if (!vehicleToTrain)
	      direction = -direction;

      if (direction == Vehicle.DIRECTION_BACKWARD and triggerName == "oil_entry3")
        return true;
      if (direction == Vehicle.DIRECTION_FORWARD and triggerName == "oil_entry0")
        return true;
      
      if (triggerName == "oil_entry0" or triggerName == "oil_entry1" or triggerName == "oil_entry2" or triggerName == "oil_entry3")
      {
        if (vehicle.GetMyTrain().IsStopped())
          return true;
      }
    }


    return false;
  }



  void PerformStoppedLoad(Vehicle vehicle, string triggerName)
  {
    if (triggerName == "fuel_loading0" or triggerName == "fuel_loading1" or triggerName == "fuel_loading2" or triggerName == "fuel_loading3")
    {
      //
      // Load the diesel
      //
      int spaceAvailable = dieselOutQueue.GetQueueCount();
      LoadingReport report = CreateLoadingReport(dieselOutQueue, spaceAvailable);

      int direction = vehicle.GetRelationToTrack(me, "outrack");
      if (direction == Vehicle.DIRECTION_FORWARD)
        report.sideFlags = LoadingReport.LEFT_SIDE;
      else if (direction == Vehicle.DIRECTION_BACKWARD)
        report.sideFlags = LoadingReport.RIGHT_SIDE;

      vehicle.LoadProduct(report);

      //
      // Load the avgas
      //
      spaceAvailable = avgasOutQueue.GetQueueCount();
      report = CreateLoadingReport(avgasOutQueue, spaceAvailable);

      direction = vehicle.GetRelationToTrack(me, "outrack");
      if (direction == Vehicle.DIRECTION_FORWARD)
        report.sideFlags = LoadingReport.LEFT_SIDE;
      else if (direction == Vehicle.DIRECTION_BACKWARD)
        report.sideFlags = LoadingReport.RIGHT_SIDE;

      vehicle.LoadProduct(report);

      //
      // Load the petrol
      //
      spaceAvailable = petrolOutQueue.GetQueueCount();
      report = CreateLoadingReport(petrolOutQueue, spaceAvailable);

      direction = vehicle.GetRelationToTrack(me, "outrack");
      if (direction == Vehicle.DIRECTION_FORWARD)
        report.sideFlags = LoadingReport.LEFT_SIDE;
      else if (direction == Vehicle.DIRECTION_BACKWARD)
        report.sideFlags = LoadingReport.RIGHT_SIDE;

      vehicle.LoadProduct(report);
    }

    if (triggerName == "oil_entry0" or triggerName == "oil_entry1" or triggerName == "oil_entry2" or triggerName == "oil_entry3")
    {
      int spaceAvailable = crudeOilInQueue.GetQueueSpace();
      LoadingReport report = CreateUnloadingReport(crudeOilInQueue, spaceAvailable);

      int direction = vehicle.GetRelationToTrack(me, "intrack");
      if (direction == Vehicle.DIRECTION_FORWARD)
        report.sideFlags = LoadingReport.LEFT_SIDE;
      else if (direction == Vehicle.DIRECTION_BACKWARD)
        report.sideFlags = LoadingReport.RIGHT_SIDE;

      vehicle.UnloadProduct(report);

      // Ensure we are tracking this if we are waiting for a way bill to complete in its entirety
      if (oilWBRemain > 0)
        oilWBRemain = oilWBRemain - report.amount;

    }

  }


 //
  //
  //
  thread void RefineryMain(void)
  {
    Message msg;
    Vehicle vehicle;
    string triggerName;

    //Interface.Print("Start of refinery thread!");

    wait()
    {
      
      on "Process-Start", "oil_refining":
      {
        SendMessage(me, "pfx", "+0+1+2+3");
        continue;
      }
      on "Process-Stop", "oil_refining":
      {
        SendMessage(me, "pfx", "-0-2-3-4");
        continue;
      }

      // ?
      on "Scriptlet-Enabled", "1":
      {
        if (!scriptletEnabled)
        {
          scriptletEnabled = true;
          SetProcessEnabled("oil_refining", true);
        }
        continue;
      }
      // ?
      on "Scriptlet-Enabled", "0":
      {
        if (scriptletEnabled)
        {
          scriptletEnabled = false;
          SetProcessEnabled("oil_refining", false);
        }
        continue;
      }


    }
  }




  //
  //
  //
  public void Init(void)
  {
    inherited();

    usePipeAnimation = true;

    // initialize queues
    crudeOilInQueue = GetQueue("in_crude_oil");
    dieselOutQueue = GetQueue("out_diesel");
    avgasOutQueue = GetQueue("out_avgas");
    petrolOutQueue = GetQueue("out_petrol");

    AddAssetToIndustryProductInfo("oil", "in_crude_oil", "oil_refining", true);
    AddAssetToIndustryProductInfo("diesel", "out_diesel", "oil_refining", false);
    AddAssetToIndustryProductInfo("petrol", "out_petrol", "oil_refining", false);
    AddAssetToIndustryProductInfo("avgas", "out_avgas", "oil_refining", false);
    
    // stop animation on refinery mesh - fueling doors are shut and not animated
    StopMeshAnimation("fuelling_doors");
    SetMeshAnimationFrame("fuelling_doors", 0);

    //Interface.Print("Initializing oil refinery here!");

    RefineryMain();

    //Interface.Print("End of oil refinery initialization!");
  }

  public Requirement[] GetRequirements(void)
  {
    Requirement[] ret = new Requirement[0];

    if (crudeOilInQueue.GetQueueCount() < 776000 or oilWBRemain > 0)  // 10 minutes till empty approx
    {
      ResourceRequirement req = new ResourceRequirement();

      req.resource = crudeOilInQueue.GetProductFilter().GetProducts()[0];
      
      // This is how many we have asked for. Wait till it is fullfilled, 
      // if we are not already waiting for a waybill to be completed.
      req.amount = 1164000;        // 30 tank cars
      if (crudeOilInQueue.GetQueueCount() < 776000 and oilWBRemain == 0)
        oilWBRemain = 1164000;

      req.dst = me;
      req.dstQueue = crudeOilInQueue;

      ret[ret.size()] = req;
    }

    return ret;
  }

  public void AppendDriverDestinations(string[] destNames, string[] destTracks)
  {
    destNames[destNames.size()] = "Oil Drop off";
    destTracks[destTracks.size()] = "intrack";
    
    destNames[destNames.size()] = "Fuel Pickup";
    destTracks[destTracks.size()] = "outrack";
    
  }

};




